package de.hft.stuttgart.citydoctor2.healer.genetic.gui;

import java.util.Objects;

import javafx.scene.image.Image;
import javafx.scene.image.PixelReader;
import javafx.scene.paint.Color;

public class ImageComparison {
	
	private ImageComparison() {
		// only static access
	}
	
	public static double compare(Image image1, Image image2) {
		assertPredefinitions(image1, image2);
		PixelReader pr1 = image1.getPixelReader();
		Objects.requireNonNull(pr1);
		PixelReader pr2 = image2.getPixelReader();
		Objects.requireNonNull(pr2);
		double pictureDif = 0;
		for (int i = 0;  i < image1.getWidth(); i++) {
			for (int j = 0; j < image2.getHeight(); j++) {
				double pixelComp = 1.0;
				Color c1 = pr1.getColor(i, j);
				Color c2 = pr2.getColor(i, j);
				double redDif = Math.abs(c1.getRed() - c2.getRed());
				double blueDif = Math.abs(c1.getBlue() - c2.getBlue());
				double greenDif = Math.abs(c1.getGreen() - c2.getGreen());
				if (redDif > 0.05) {
					pixelComp -= 1/3d;
				}
				if (blueDif > 0.05) {
					pixelComp -= 1/3d;
				}
				if (greenDif > 0.05) {
					pixelComp -= 1/3d;
				}
				pictureDif += pixelComp;
			}
		}
		double dimension = image1.getWidth() * image1.getHeight();
		pictureDif = Math.max(0, dimension - 50 * (dimension - pictureDif));
		return pictureDif / dimension;
	}

	private static void assertPredefinitions(Image image1, Image image2) {
		if (image1.getProgress() != 1.0 || image2.getProgress() != 1.0) {
			throw new IllegalArgumentException("Pictures are not fully loaded");
		}
		if (image1.getWidth() != image2.getWidth() || image1.getHeight() != image2.getHeight()) {
			throw new IllegalArgumentException("Pictures have to be the same size");
		}
	}

}
